from django.db.models import Count, CharField, Value, When, Case

from simulations.models import Individual, Population, Generation, Experiment
from rest_framework import routers, serializers, viewsets


class IndividualSerializer(serializers.ModelSerializer):
    class Meta:
        model = Individual
        fields = ("id", "connections", "color")

    color = serializers.SerializerMethodField()

    connections = serializers.ManyRelatedField(
        child_relation=serializers.PrimaryKeyRelatedField(
            queryset=Individual.objects.all()
        )
    )

    def get_color(self, obj):
        # The color is a function of the individual being infected or not
        return "red" if obj.pathogen else "green"


class PopulationSerializer(serializers.ModelSerializer):
    class Meta:
        model = Population
        fields = "__all__"

    # individuals = IndividualSerializer(many=True)
    nodes = serializers.SerializerMethodField()
    links = serializers.SerializerMethodField()

    def get_links(self, obj):
        links = []

        annotated = obj.individuals.annotate(connections_count=Count("connections"))
        connected = annotated.filter(connections_count__gt=0)

        # Can we speed this up?
        for individual in connected:
            for connection in individual.connections.all():
                links.append((individual.id, connection.id))

        return links

    def get_nodes(self, obj):
        nodes = []
        individuals = obj.individuals.all()
        individuals = individuals.annotate(
            color=Case(
                When(pathogen__isnull=False, then=Value("red")),
                default=Value("green"),
                output_field=CharField(),
            )
        )

        for individual in individuals:
            nodes.append({"id": individual.id, "genotype": individual.genotype, "color": individual.color})

        return nodes


class GenerationSerializer(serializers.ModelSerializer):
    class Meta:
        model = Generation
        fields = "__all__"

    population = PopulationSerializer()