import random
from django.core.management.base import BaseCommand, CommandError
from simulations.models import Simulation, Population, Individual, Pathogen
from simulations.network_strategies import generate_small_world_population
from simulations.visualization import visualize_population_no_weights


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument("count", default=1000, type=int)

    def handle(self, *args, **options):
        count = options["count"]
        simulation = Simulation.objects.create()
        population = Population.objects.create(
            network_topology=Population.NetworkTopology.SMALL_WORLD,
            simulation=simulation,
        )

        self.stdout.write(
            self.style.SUCCESS(
                f"Created simulation {simulation.id} with population {population.id}"
            )
        )

        self.stdout.write(self.style.SUCCESS(f"Generating {count} individuals"))
        Individual.objects.bulk_create(
            [
                Individual(
                    population=population,
                    genotype="".join([str(random.randint(0, 1)) for _ in range(4)]),
                )
                for _ in range(count)
            ]
        )

        self.stdout.write(self.style.SUCCESS(f"Generating scale free network"))
        generate_small_world_population(population)

        img = visualize_population_no_weights(population)
        img.save("small_world.png")

        self.stdout.write(
            self.style.SUCCESS(
                f"Generated {population.individuals.count()} individuals"
            )
        )
