def generation_statistics(generation):
    from .models import Individual, Pathogen
    from django.db.models import ExpressionWrapper, F, IntegerField

    computed_incubation_time = ExpressionWrapper(
        F('pathogen__incubation_time_points') / 100.0 * Pathogen.MAXIMUM_TIME +
        Pathogen.MINIMUM_TIME * (1 - F('pathogen__incubation_time_points') / 100.0),
        output_field=IntegerField()
    )

    computed_infection_time = ExpressionWrapper(
        F('pathogen__infection_time_points') / 100.0 * Pathogen.MAXIMUM_TIME +
        Pathogen.MINIMUM_TIME * (1 - F('pathogen__infection_time_points') / 100.0),
        output_field=IntegerField()
    )

    individuals = generation.individuals.all().annotate(
        computed_incubation_time=computed_incubation_time,
        computed_infection_time=computed_infection_time,
    )

    total_population = individuals.count()
    alive = individuals.filter(state=Individual.StateOptions.ALIVE)
    alive_count = alive.count()
    dead = individuals.filter(state=Individual.StateOptions.DEAD).count()

    alive_infected = alive.filter(pathogen__isnull=False)
    alive_infected_count = alive_infected.count()
    alive_not_infected = alive.filter(pathogen__isnull=True)
    alive_not_infected_count = alive_not_infected.count()

    incubating = alive_infected.filter(
        pathogen__days_infected__lte=F("computed_incubation_time")
    )
    sick = alive_infected.filter(
        pathogen__days_infected__gt=F("computed_incubation_time"),
        pathogen__days_infected__lte=F("computed_incubation_time")
        + F("computed_infection_time"),
    )

    recovered = alive_infected.filter(
        pathogen__days_infected__gt=F("computed_incubation_time")
        + F("computed_infection_time")
    )

    return {
        "total_population": total_population,
        "alive": alive_count,
        "dead": dead,
        "alive_infected": alive_infected_count,
        "alive_not_infected": alive_not_infected_count,
        "incubating": incubating.count(),
        "sick": sick.count(),
        "recovered": recovered.count(),
    }
