import random


def re_allocate_skill_points(pathogen):
    "Performs a re-allocation in small increments. Picks a random trait and re-allocates 5 or 10 points to another"
    from simulations.models import Pathogen
    import random

    possible_trait_values = [5, 10]
    possible_traits = [
        "death_rate_points",
        "infection_rate_points",
        "incubation_time_points",
        "infection_time_points",
    ]

    # Initial trait and value selection
    trait_to_change = random.choice(possible_traits)
    value_to_change_by = random.choice(possible_trait_values)
    trait_to_change_to = random.choice(possible_traits)

    # Get the current value of the traits
    current_trait_value = getattr(pathogen, trait_to_change)
    current_trait_value_to_change_to = getattr(pathogen, trait_to_change_to)

    # If the trait to change has enough points to be decreased and the trait to change to can lose points without going negative
    if (
        current_trait_value > value_to_change_by
        and current_trait_value_to_change_to >= value_to_change_by
    ):
        setattr(pathogen, trait_to_change, current_trait_value - value_to_change_by)
        setattr(
            pathogen,
            trait_to_change_to,
            current_trait_value_to_change_to + value_to_change_by,
        )

    # If the trait to change doesn't have enough points to be decreased, but can gain points and the trait to change to can lose points without going negative
    elif (
        current_trait_value < value_to_change_by
        and current_trait_value + value_to_change_by <= 100
        and current_trait_value_to_change_to >= value_to_change_by
    ):
        setattr(pathogen, trait_to_change, current_trait_value + value_to_change_by)
        setattr(
            pathogen,
            trait_to_change_to,
            current_trait_value_to_change_to - value_to_change_by,
        )

    return pathogen


def sample_and_mutate_pathogen_qs(pathogen_qs, mutation_rate=0.01):
    offspring = []
    for pathogen in pathogen_qs:
        pathogen.pk = None
        pathogen.generation = None
        pathogen = re_allocate_skill_points(pathogen)
        pathogen.save()
        offspring.append(pathogen)

    return pathogen_qs


def tournament_selection(pathogens, tournament_size=3):
    from simulations.fitness import calculate_pathogen_score

    # Randomly select a few individuals for the tournament
    tournament = random.sample(pathogens, tournament_size)

    # Select the best individual from the tournament
    winner = max(tournament, key=calculate_pathogen_score)
    return winner


def single_point_crossover(parent1, parent2):
    from simulations.models import Pathogen

    assert isinstance(parent1, Pathogen), "parent1 must be an instance of Pathogen"
    assert isinstance(parent2, Pathogen), "parent2 must be an instance of Pathogen"

    # List of attributes for crossover
    attributes = [
        "death_rate_points",
        "immunity_duration_points",
        "infection_rate_points",
        "incubation_time_points",
        "infection_time_points",
        "mutation_rate_points",
    ]

    # Choose a random crossover point
    crossover_point = random.randint(1, len(attributes) - 1)
    crossover_point_av = random.randint(1, len(parent1.attack_vector) - 1)

    # Create children
    child1_data = {}
    child2_data = {}

    for i, field in enumerate(attributes):
        if i < crossover_point:
            child1_data[field] = getattr(parent1, field)
            child2_data[field] = getattr(parent2, field)
        else:
            child1_data[field] = getattr(parent2, field)
            child2_data[field] = getattr(parent1, field)

    child1_data["attack_vector"] = (
        parent1.attack_vector[:crossover_point_av]
        + parent2.attack_vector[crossover_point_av:]
    )
    child2_data["attack_vector"] = (
        parent2.attack_vector[:crossover_point_av]
        + parent1.attack_vector[crossover_point_av:]
    )

    # Create the new Pathogen objects
    child1 = Pathogen.objects.create(**child1_data)
    child2 = Pathogen.objects.create(**child2_data)

    return child1, child2


def crossover(parent1, parent2):
    from simulations.models import Pathogen

    # A simple crossover function could be averaging the values of the parents.
    child = Pathogen()

    # Note that here we are considering only three attributes for simplification, you need to add all the attributes of your Pathogen model
    child.attack_vector = int((parent1.attack_vector + parent2.attack_vector) / 2)
    child.death_rate_points = int(
        (parent1.death_rate_points + parent2.death_rate_points) / 2
    )
    child.immunity_duration_points = int(
        (parent1.immunity_duration_points + parent2.immunity_duration_points) / 2
    )

    return child


def mutate(pathogen, mutation_rate):
    # Here, we're adding or subtracting 1 to a random attribute. You may want to come up with a more sophisticated mutation function.
    attributes = [
        "death_rate_points",
        "immunity_duration_points",
        "infection_rate_points",
        "incubation_time_points",
        "infection_time_points",
    ]
    if random.random() < mutation_rate:
        attribute_to_increase, attribute_to_decrease = random.sample(attributes, 2)

        # Add mutation points to the chosen attribute, prevent values above the maximum allowed
        old_value = getattr(pathogen, attribute_to_increase)
        new_value = min(old_value + 5, 100)
        setattr(pathogen, attribute_to_increase, new_value)
        points_added = (
            new_value - old_value
        )  # Store the actual points added, in case we hit the maximum

        # Subtract the same amount of points from the other chosen attribute, prevent negative values
        old_value = getattr(pathogen, attribute_to_decrease)
        new_value = max(old_value - points_added, 0)
        setattr(pathogen, attribute_to_decrease, new_value)

    return pathogen
