

def compute_graph_statistics(graph):
    import networkx as nx

    statistics = {
        "number_of_nodes": nx.number_of_nodes(graph),
        "number_of_edges": nx.number_of_edges(graph),
        "average_degree": sum(dict(graph.degree()).values())
        / nx.number_of_nodes(graph),
        "average_clustering_coefficient": nx.average_clustering(graph),
        "average_shortest_path_length": nx.average_shortest_path_length(graph)
        if nx.is_connected(graph)
        else "Graph is not connected",
        "diameter": nx.diameter(graph)
        if nx.is_connected(graph)
        else "Graph is not connected",
        "degree_centrality": nx.degree_centrality(graph),
        "betweenness_centrality": nx.betweenness_centrality(graph),
        "closeness_centrality": nx.closeness_centrality(graph),
        "eigenvector_centrality": 0, # nx.eigenvector_centrality(graph),
        "average_eigenvector_centrality": 0, # sum(nx.eigenvector_centrality(graph).values()) / nx.number_of_nodes(graph),
        "node_connectivity": nx.node_connectivity(graph),
        "edge_connectivity": nx.edge_connectivity(graph),
    }

    # If the graph is not too large, you can compute the full centrality measures
    # and convert them to averages, else you can comment these lines out.
    statistics["average_degree_centrality"] = sum(
        statistics["degree_centrality"].values()
    ) / len(statistics["degree_centrality"])
    statistics["average_betweenness_centrality"] = sum(
        statistics["betweenness_centrality"].values()
    ) / len(statistics["betweenness_centrality"])
    statistics["average_closeness_centrality"] = sum(
        statistics["closeness_centrality"].values()
    ) / len(statistics["closeness_centrality"])
   # statistics["average_eigenvector_centrality"] = sum(
    #    statistics["eigenvector_centrality"].values()
    #) / len(statistics["eigenvector_centrality"])

    return statistics
