from django.test import TestCase
from simulations.models import Pathogen
from simulations.sampling import single_point_crossover
from simulations.skill_distribution import random_skill_distribution
from tabulate import tabulate


class TestCrossover(TestCase):
    def setUp(self):
        kwargs1 = random_skill_distribution()
        kwargs2 = random_skill_distribution()
        self.parent1 = Pathogen.objects.create(
            attack_vector="1001010111",
            **kwargs1,
        )
        self.parent2 = Pathogen.objects.create(
            attack_vector="0101101001",
            **kwargs2,
        )

    def test_single_point_crossover(self):
        child1, child2 = single_point_crossover(self.parent1, self.parent2)

        attributes = [
            "attack_vector",
            "death_rate_points",
            "infection_rate_points",
            "incubation_time_points",
            "infection_time_points",
            "mutation_rate_points",
            "immunity_duration_points",
        ]

        entities = [
            ("Parent 1", self.parent1),
            ("Parent 2", self.parent2),
            ("Child 1", child1),
            ("Child 2", child2),
        ]

        table_data = []
        for name, entity in entities:
            # Get attribute values and sum of points
            attr_values = [getattr(entity, attr) for attr in attributes]
            points_sum = sum(attr_values[1:])  # Assuming 'attack_vector' is not a numerical value
            row_data = [name] + attr_values + [points_sum]
            table_data.append(row_data)

        # Add 'Total Points' to headers
        headers = ['Name'] + attributes + ['Total Points']
        print(tabulate(table_data, headers, tablefmt='grid'))
