from django.test import TestCase
from simulations.models import Pathogen
from simulations.skill_distribution import random_skill_distribution


class RandomSkillDistributionTest(TestCase):
    def test_distribution_sum(self):
        for i in range(1000):
            attribute_dict = random_skill_distribution()
            self.assertEqual(sum(attribute_dict.values()), 100)

    def test_distribution_values(self):
        attribute_dict = random_skill_distribution()
        for value in attribute_dict.values():
            self.assertGreaterEqual(value, 0)
            self.assertLessEqual(value, 100)

    def test_distribution_with_pathogen_creation(self):
        attribute_dict = random_skill_distribution()
        pathogen = Pathogen.objects.create(**attribute_dict)
        self.assertIsNotNone(pathogen)
        self.assertEqual(pathogen.infection_rate_points, attribute_dict['infection_rate_points'])
        self.assertEqual(pathogen.death_rate_points, attribute_dict['death_rate_points'])
        self.assertEqual(pathogen.mutation_rate_points, attribute_dict['mutation_rate_points'])
        self.assertEqual(pathogen.incubation_time_points, attribute_dict['incubation_time_points'])
        self.assertEqual(pathogen.infection_time_points, attribute_dict['infection_time_points'])
        self.assertEqual(pathogen.immunity_duration_points, attribute_dict['immunity_duration_points'])
